// 注册自定义的 maxlength 模块
Quill.register('modules/maxlength', function(instance, options) {
    if (!('value' in options)) {
        return;
    }
    instance.on('text-change', () => {
        const text = instance.getText();
        if (text.length > options.value) {
            instance.history.undo();
        }
    });
});

var quill = new Quill('#editor-container', {
    theme: 'snow',
    placeholder: '请输入备注',
    modules: {
        // toolbar: false  // 不启用工具栏
        maxlength: { value: 501 }, // 默认自带\n
        toolbar: [
            ['bold'],
            [{ 'list': 'ordered' }, { 'list': 'bullet' },{ 'list': 'check' }],
        ]
    }
});

// 去掉底部/左/右边框
document.querySelector('.ql-container').style.borderBottom = 'none';
document.querySelector('.ql-container').style.borderLeft = 'none';
document.querySelector('.ql-container').style.borderRight = 'none';


var selectionIndex = -1; //选中文本的位置
var selectionLength = 0;//选中文本的长度
var globalBold = false //全局加粗状态
var timeoutId = -1

var readOnly = false

var parentNode;
// 监听 check 点击事件
document.querySelector('#editor-container').addEventListener('click', function (event) {
    let target = event.target;
    if (target.tagName === 'SPAN' && target.classList.contains('ql-ui')) {
        event.stopPropagation(); // 阻止事件冒泡
        let parent = target.parentNode;
        parentNode = parent
        postMessage(JSON.stringify(buildClickEventMessage()))
    }
});

function setConfirmCheck(source){
    let checked = false;
    let parent = parentNode;
    if (parent.dataset.list === 'checked') {
        parent.dataset.list = 'unchecked';
        checked = false;
    } else if (parent.dataset.list === 'unchecked') {
        parent.dataset.list = 'checked';
        checked = true;
    }
    if(readOnly){//只读模式下，需要先开启一下，detla内容才会变化
        quill.enable()
        setTimeout(function () {//继续只读模式
            quill.disable()
        }, 50);
    }
    changeCheckListStyle()
    postMessage(JSON.stringify(buildCheckboxChangeMessage(checked,source)))
}



// 监听 checked/unecheked
let listItems = document.querySelector(".ql-editor").querySelectorAll("li");
listItems.forEach(li => {
    let isChecked = li.getAttribute("data-list") === "checked";
    if (isChecked) {
        li.style.textDecoration = "line-through"; // 加删除线
        li.style.color = "#8E99A1"; // 修改颜色（与删除线一致）
    } else {
        li.style.textDecoration = "none"; // 取消删除线
        li.style.color = ""; // 还原默认颜色
    }
});



// 监听内容高度
quill.on('text-change', function(delta) {
    if(quill.getLength() <= 1){//内容清空时，通知toolbar恢复初始状态
        var format = quill.getFormat();
        postMessage(JSON.stringify(buildFontMessage(false)))
        postMessage(JSON.stringify(buildListMessage(format.list)))
        postMessage(JSON.stringify(buildContentMessage('','')))
        return
    }


//    delta.ops.forEach(function (op) {
//        if (op.insert === '\n') {
//            console.log('发生了换行');
//            clearSelection();
//            timeoutId = setTimeout(function () {
//                console.log('text-change setTimeout after: ', JSON.stringify(quill.getFormat()));
//                callbackCurrentFormat();
//            }, 50);
//        } else if (op.insert) {
//            clearSelection();
//            console.log('插入了文本：', op.insert);
//            timeoutId = setTimeout(function () {
//                const selection = quill.getSelection();
//                console.log('text-change ','selection:'+JSON.stringify(selection) +",globalBold:"+globalBold  + " ,op.insert.length:"+op.insert.length);
//                // 确保光标位置是有效的，并且光标后有内容
//                if (selection && selection.index >= 0) {
//                    quill.formatText(selection.index-op.insert.length, op.insert.length, {'bold': globalBold});
//                }
//            }, 50);
//        } else if (op.delete) {
//            clearSelection();
//            console.log('删除了文本：', op.delete);
//            timeoutId = setTimeout(function () {
//                console.log('text-change setTimeout after: ', JSON.stringify(quill.getFormat()));
//                callbackCurrentFormat();
//            }, 50);
//        }else {
//            console.log('else '+JSON.stringify(op));
//        }
//    });

    //清除倒计时
    if(timeoutId != -1){
        clearTimeout(timeoutId);
    }
    //这里需要延时，否则 换行后，拿到的format 是上一行的（例如：上一行末尾有加粗，换行后 加粗按钮应该恢复非加粗按钮）
    timeoutId = setTimeout(function () {
        console.log('text-change setTimeout after: ', JSON.stringify(quill.getFormat()));
        callbackCurrentFormat();
        postMessage(JSON.stringify(buildContentMessage(getEditorContent(),quill.getText())))

        let height = document.querySelector('.ql-editor').scrollHeight;
        postMessage(JSON.stringify(buildLayoutMessage(height)));
    }, 50);

    changeCheckListStyle()

});

//监听光标位置变化
quill.on('selection-change', (range, oldRange, source) => {
    console.log(`User cursor is on range:${JSON.stringify(range)} oldRange:${JSON.stringify(oldRange)} source:${source}`)
    if (range) {
        if (range.length == 0) {
            console.log('User cursor is on', range.index);
            selectionLength = 0
        } else {
            const text = quill.getText(range.index, range.length);
            console.log('User has highlighted', text);
            selectionIndex = range.index;
            selectionLength = range.length;
        }
        callbackCurrentFormat();
    } else {
        console.log('Cursor not in the editor');
    }
});

function changeCheckListStyle() {
    requestAnimationFrame(() => {
        let editor = document.querySelector(".ql-editor");
        let listItems = editor.querySelectorAll("li");

        listItems.forEach(li => {
            let isChecked = li.getAttribute("data-list") === "checked";
            if (isChecked) {
                li.style.textDecoration = "line-through"; // 加删除线
                li.style.color = "#8E99A1"; // 修改颜色（与删除线一致）
            } else {
                li.style.textDecoration = "none"; // 取消删除线
                li.style.color = ""; // 还原默认颜色
            }
        });
    });
}



function callbackCurrentFormat() {
    // 获取光标的当前格式
    let format = quill.getFormat();
    if(format){
        globalBold = format.bold??false
        console.log('callbackCurrentFormat globalBold:'+ globalBold + " ,list:"+ format.list);
        postMessage(JSON.stringify(buildFontMessage(globalBold)));
        postMessage(JSON.stringify(buildListMessage(format.list)));
    }
}

function setListBullet() {
    var format = quill.getFormat();
    if (format.list === 'bullet') {
        quill.format('list', false); // 取消列表
    } else {
        quill.format('list', 'bullet');
    }
}

function setListCheck() {
    var format = quill.getFormat();
    if (format.list === 'unchecked'||format.list === 'checked') {
        quill.format('list', false); // 取消checkbox
    } else {
        quill.format('list', 'unchecked');
    }
}

function buildFontMessage(bold) {
    let message = {
        type:'font',
        bold:bold
    };
    return message;
}

function buildLayoutMessage(height) {
    let message = {
        type:'layout',
        height:height
    };
    return message;
}

function buildListMessage(list) {
    let message = {
        type:'list',
        list:list
    }
    return message;
}

function buildContentMessage(fullMessage,simpleMessage) {
    let message = {
        type:'content',
        full:fullMessage,
        simple:simpleMessage
    }
    return message;
}

function buildCheckboxChangeMessage(checked,source) {
    let message = {
        type:'checkbox',
        checked:checked,
        source:source
    }
    return message;
}

function buildClickEventMessage() {
    let message = {
        type:'click'
    }
    return message;
}

function clearSelection() {
    selectionIndex = -1
    selectionLength = 0
}



// Function to get editor content as Delta JSON
function getEditorContent() {
    return JSON.stringify(quill.getContents());
}

// Function to set editor content from Delta JSON
function setEditorContent(content) {
    try {
        const delta = JSON.parse(content);
        quill.setContents(delta);
    } catch (error) {
        console.log("Invalid Delta JSON:", error);
    }
}

function setQuillReadOnly() {
    readOnly = true
    quill.disable();
    //禁止滑动
    document.querySelector('.ql-editor').style.overflow = 'hidden'
}

function setFocus(){
    quill.focus();
    const length = quill.getLength();
    quill.setSelection(length,0);//光标在最后
    quill.scrollSelectionIntoView();

}



function setBold(isBold) {
    console.log('setBold isBold:'+ isBold +',selectionIndex:'+selectionIndex + ",selectionLength:"+selectionLength);
    if(selectionIndex != -1 && selectionLength > 0){
        var format = quill.getFormat(selectionIndex, selectionLength);
        var bold = false
        if(format){
            bold = !format.bold
        }
        quill.formatText(selectionIndex, selectionLength, 'bold', bold);
        globalBold = bold
    }else{
        if(isBold == 'false'){
            globalBold = false
            quill.format('bold', false)
        }else if(isBold == 'true'){
            globalBold = true
            quill.format('bold', true)
        }

    }
}

function postMessage(json){
    Android.callbackMessage(json);
}